/* Scrutari - Copyright (c) 2008-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.ResourceReference;
import fr.exemole.sctserver.api.EngineContext;
import fr.exemole.sctserver.api.SctEngine;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.docstream.ClassResourceDocStream;
import net.mapeadores.util.mimetype.MimeTypeConstants;
import net.mapeadores.util.mimetype.MimeTypeUtils;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.DocStreamResponseHandler;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public final class ResourceResponseHandlerFactory {

    private ResourceResponseHandlerFactory() {
    }

    public static ResponseHandler getHandler(SctEngine sctEngine, String path) {
        RelativePath relativePath = RelativePath.newInstance(path);
        if (relativePath == null) {
            return null;
        }
        return getHandler(sctEngine, relativePath);
    }

    public static ResponseHandler getHandler(SctEngine sctEngine, RelativePath relativePath) {
        DocStream docStream = sctEngine.getEngineContext().getResourceDocStream(relativePath, sctEngine);
        if (docStream != null) {
            return new DocStreamResponseHandler(docStream);
        }
        return getHandler(sctEngine.getEngineContext(), relativePath);
    }

    public static ResponseHandler getHandler(EngineContext engineContext, String path) {
        RelativePath relativePath = RelativePath.newInstance(path);
        if (relativePath == null) {
            return null;
        }
        return getHandler(engineContext, relativePath);
    }

    public static ResponseHandler getHandler(EngineContext engineContext, RelativePath relativePath) {
        DocStream docStream = engineContext.getResourceDocStream(relativePath, null);
        if (docStream != null) {
            return new DocStreamResponseHandler(docStream);
        }
        ClassResourceDocStream classResourceDocStream = ClassResourceDocStream.newInstance(ResourceReference.class, "resources/" + relativePath.toString());
        if (classResourceDocStream != null) {
            String mimeType = MimeTypeUtils.getMimeType(MimeTypeUtils.DEFAULT_RESOLVER, relativePath.getFileName());
            classResourceDocStream.setMimeType(mimeType);
            return new DocStreamResponseHandler(classResourceDocStream);
        } else {
            return null;
        }
    }


    private static DocStream getCssDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/theme/css/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.CSS);
        return docStream;
    }

    private static DocStream getJsDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/js/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType(MimeTypeConstants.JAVASCRIPT);
        return docStream;
    }

    private static DocStream getPngDocStream(String fileName) {
        ClassResourceDocStream docStream = getDocStream("resources/theme/png/" + fileName);
        if (docStream == null) {
            return null;
        }
        docStream.setMimeType("image/png");
        return docStream;
    }

    private static ClassResourceDocStream getDocStream(String resourcePath) {
        return ClassResourceDocStream.newInstance(ResourceReference.class, resourcePath);
    }

}
