/* Scrutari - Copyright (c) 2009-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.admin.htmlpages.utils;

import fr.exemole.sctserver.servlets.admin.htmlpages.AdminHtmlPage;
import java.util.List;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.text.Label;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;


/**
 *
 * @author Vincent Calame
 */
public final class CategoriesPrint {

    private CategoriesPrint() {
    }

    public static void printCategories(AdminHtmlPage hp) {
        DataAccess dataAccess = hp.getEngine().getScrutariSession().getScrutariDB().openDataAccess();
        GlobalSearchOptions globalSearchOptions = hp.getEngine().getScrutariSession().getGlobalSearchOptions();
        List<Category> categoryList = globalSearchOptions.getCategoryList();
        if (categoryList.isEmpty()) {
            hp
                    .P()
                    .SPAN(HA.classes("scrutari-admin-Info"))
                    .localize("_ info.sctconf.none_categories")
                    ._SPAN()
                    ._P();
        } else {
            hp
                    .UL();
            for (Category category : categoryList) {
                printCategoryLi(hp, category.getCategoryDef(), dataAccess);
            }
            hp
                    ._UL();
        }
        dataAccess.close();
    }

    private static void printCategoryLi(AdminHtmlPage hp, CategoryDef categoryDef, DataAccess dataAccess) {
        HtmlAttributes errorAttributes = HA.classes("scrutari-admin-State_Error");
        List<CorpusURI> corpusURIList = categoryDef.getCorpusURIList();
        String name = categoryDef.getName();
        hp
                .LI();
        hp
                .P()
                .CODE()
                .escape('[')
                .append(categoryDef.getRank())
                .escape(" / ")
                .escape(name)
                .escape(']')
                ._CODE()
                .escape(" - ");
        int labelCount = categoryDef.getLabelCount();
        for (int i = 0; i < labelCount; i++) {
            if (i > 0) {
                hp
                        .escape(" / ");
            }
            Label label = categoryDef.getLabelByIndex(i);
            hp
                    .escape(label.getLabelString());
        }
        hp
                ._P();
        hp
                .UL();
        if (corpusURIList.isEmpty()) {
            hp
                    .LI()
                    .P()
                    .EM();
            if (name.equals("_default")) {
                hp
                        .localize("_ info.sctconf.defaultcorpusuri");
            } else {
                hp
                        .localize("_ info.sctconf.none_corpusuri");
            }
            hp
                    ._EM()
                    ._P()
                    ._LI();
        } else {
            for (CorpusURI corpusURI : corpusURIList) {
                int codecorpus = dataAccess.getCode(corpusURI);
                boolean isHere = (codecorpus > 0);
                hp
                        .LI();
                hp
                        .P();
                if (!isHere) {
                    hp
                            .SPAN(errorAttributes);
                }
                hp
                        .CODE()
                        .escape('[')
                        .escape(corpusURI)
                        .escape(']')
                        ._CODE();
                if (!isHere) {
                    hp
                            ._SPAN();
                }
                hp
                        ._P()
                        ._LI();
            }
        }
        hp
                ._UL();
        hp
                ._LI();
    }

}
