/* Scrutari - Copyright (c) 2012-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets.export;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import fr.exemole.sctserver.servlets.export.contentstreams.ResultOdsStream;
import java.text.ParseException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.result.FicheSearchResult;


/**
 *
 * @author Vincent Calame
 */
public final class ExportResponseHandlerFactory {

    private ExportResponseHandlerFactory() {

    }

    public static ResponseHandler getHandler(SctEngine engine, String pathInfo, RequestMap requestMap) {
        if ((pathInfo.startsWith("/result_")) && (pathInfo.endsWith(".ods"))) {
            ScrutariSession scrutariSession = engine.getScrutariSession();
            ResultParameters resultParameters = initResultParameters(scrutariSession, pathInfo.substring(8, pathInfo.length() - 4));
            if (resultParameters == null) {
                return null;
            } else {
                return new ResultOdsStream(scrutariSession, resultParameters.ficheSearchResult, resultParameters.langUi);
            }
        } else {
            return null;
        }
    }

    private static ResultParameters initResultParameters(ScrutariSession scrutariSession, String param) {
        int idx2 = param.lastIndexOf('_');
        Lang langUi;
        try {
            langUi = Lang.parse(param.substring(idx2 + 1));
        } catch (ParseException pe) {
            return null;
        }
        FicheSearchResult ficheSearchResult = null;
        try {
            QId qId = QId.parse(param.substring(0, idx2));
            ficheSearchResult = scrutariSession.getSearch(qId);
        } catch (ParseException pe) {
        }
        if (ficheSearchResult == null) {
            return null;
        }
        return new ResultParameters(ficheSearchResult, langUi);
    }


    private static class ResultParameters {

        private final FicheSearchResult ficheSearchResult;
        private final Lang langUi;

        private ResultParameters(FicheSearchResult ficheSearchResult, Lang langUi) {
            this.ficheSearchResult = ficheSearchResult;
            this.langUi = langUi;
        }

    }

}
