/* ScrutariLib_DB - Copyright (c) 2007-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools.util;

import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;


/**
 *
 * @author Vincent Calame
 */
public final class DataLabelUtils {

    private DataLabelUtils() {
    }

    public static String checkLabelString(CorpusData corpusData, int fieldNumber, Lang lang, String defaultString) {
        CorpusData.CorpusLangData corpusLangData = corpusData.getLangDataByLang(lang);
        String labelString = DataLabelUtils.getLabelString(corpusLangData, fieldNumber);
        if (labelString != null) {
            return labelString;
        }
        if (!lang.isRootLang()) {
            labelString = DataLabelUtils.getLabelString(corpusData.getLangDataByLang(lang.getRootLang()), fieldNumber);
            if (labelString != null) {
                return labelString;
            }
        }
        int count = corpusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            corpusLangData = corpusData.getLangDataByIndex(i);
            labelString = DataLabelUtils.getLabelString(corpusLangData, fieldNumber);
            if (labelString != null) {
                return labelString;
            }
        }
        return defaultString;
    }

    public static String checkLabelString(BaseData baseData, short type, Lang lang, String defaultString) {
        BaseData.BaseLangData baseLangData = baseData.getLangDataByLang(lang);
        String labelString = DataLabelUtils.getLabelString(baseLangData, type);
        if (labelString != null) {
            return labelString;
        }
        if (!lang.isRootLang()) {
            labelString = DataLabelUtils.getLabelString(baseData.getLangDataByLang(lang.getRootLang()), type);
            if (labelString != null) {
                return labelString;
            }
        }
        int count = baseData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            baseLangData = baseData.getLangDataByIndex(i);
            labelString = DataLabelUtils.getLabelString(baseLangData, type);
            if (labelString != null) {
                return labelString;
            }
        }
        return defaultString;
    }

    public static String checkLabelString(ThesaurusData thesaurusData, Lang lang, String defaultString) {
        ThesaurusData.ThesaurusLangData thesaurusLangData = thesaurusData.getLangDataByLang(lang);
        String res = getLabelString(thesaurusLangData);
        if (res != null) {
            return res;
        }
        if (!lang.isRootLang()) {
            res = getLabelString(thesaurusData.getLangDataByLang(lang.getRootLang()));
            if (res != null) {
                return res;
            }
        }
        int count = thesaurusData.getLangDataCount();
        for (int i = 0; i < count; i++) {
            thesaurusLangData = thesaurusData.getLangDataByIndex(i);
            res = getLabelString(thesaurusLangData);
            if (res != null) {
                return res;
            }
        }
        return defaultString;
    }

    private static String getLabelString(CorpusData.CorpusLangData corpusLangData, int fieldNumber) {
        if (corpusLangData == null) {
            return null;
        }
        Label label = corpusLangData.getLabel(fieldNumber);
        if (label == null) {
            return null;
        }
        return label.getLabelString();
    }

    private static String getLabelString(BaseData.BaseLangData baseLangData, short type) {
        if (baseLangData == null) {
            return null;
        }
        Label label = baseLangData.getLabel(type);
        if (label == null) {
            return null;
        }
        return label.getLabelString();
    }

    private static String getLabelString(ThesaurusData.ThesaurusLangData thesaurusLangData) {
        if (thesaurusLangData == null) {
            return null;
        }
        Label label = thesaurusLangData.getLabel();
        if (label == null) {
            return null;
        }
        return label.getLabelString();
    }

}
