/* SctServer - Copyright (c) 2010-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request.json;

import fr.exemole.sctserver.json.FicheDataJson;
import fr.exemole.sctserver.request.FicheFieldsFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.json.JSONWriter;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.BaseData;
import net.scrutari.data.FicheData;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.Codes;
import net.scrutari.db.tools.util.BaseCheck;


/**
 *
 * @author Vincent Calame
 */
public class FicheJsonProducer extends AbstractJsonProducer {

    private final int version;
    private final ScrutariDB scrutariDB;
    private final Codes ficheCodes;
    private final FicheFieldsFilter ficheFieldsFilter;
    private final Lang langUi;

    public FicheJsonProducer(int version, ScrutariDB scrutariDB, Codes ficheCodes, FicheFieldsFilter ficheFieldsFilter, Lang langUi) {
        if (langUi == null) {
            throw new IllegalArgumentException("langUi is null");
        }
        this.version = version;
        this.scrutariDB = scrutariDB;
        this.ficheCodes = ficheCodes;
        this.ficheFieldsFilter = ficheFieldsFilter;
        this.langUi = langUi;
    }

    @Override
    public void writeJson(Appendable appendable) throws IOException {
        DataAccess dataAccess = scrutariDB.openDataAccess();
        JSONWriter jsonWriter = new JSONWriter(appendable);
        jsonWriter.object();
        if (version > 1) {
            printVersion2(jsonWriter, dataAccess);
        } else {
            printVersion1(jsonWriter, dataAccess);
        }
        writeWarnings(jsonWriter);
        jsonWriter.endObject();
        dataAccess.close();
    }

    private void printVersion2(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        int ficheCount = ficheCodes.getCodeCount();
        jsonWriter.key("ficheArray");
        jsonWriter.array();
        for (int i = 0; i < ficheCount; i++) {
            int codefiche = ficheCodes.getCode(i);
            FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
            BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, langUi);
            jsonWriter.object();
            FicheDataJson.object(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, null, scrutariDB.getAttributeKeyCoder());
            jsonWriter.endObject();
        }
        jsonWriter.endArray();
    }

    private void printVersion1(JSONWriter jsonWriter, DataAccess dataAccess) throws IOException {
        Map<Integer, BaseCheck> bufferMap = new HashMap<Integer, BaseCheck>();
        jsonWriter.key("ficheData");
        jsonWriter.object();
        int ficheCount = ficheCodes.getCodeCount();
        jsonWriter.key("ficheCount");
        jsonWriter.value(ficheCount);
        if (ficheCount > 0) {
            jsonWriter.key("ficheArray");
            jsonWriter.array();
            for (int i = 0; i < ficheCount; i++) {
                int codefiche = ficheCodes.getCode(i);
                FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                BaseCheck baseCheck = getBaseCheck(dataAccess, ficheData.getCodeBase(), bufferMap, langUi);
                jsonWriter.object();
                FicheDataJson.object(jsonWriter, ficheData, ficheFieldsFilter, baseCheck, null, scrutariDB.getAttributeKeyCoder());
                jsonWriter.endObject();
            }
            jsonWriter.endArray();
        }
        jsonWriter.endObject();
    }

    private BaseCheck getBaseCheck(DataAccess dataAccess, int codebase, Map<Integer, BaseCheck> bufferMap, Lang langUi) {
        BaseCheck baseCheck = bufferMap.get(codebase);
        if (baseCheck != null) {
            return baseCheck;
        }
        BaseData baseData = dataAccess.getBaseDataByCode(codebase);
        baseCheck = BaseCheck.check(baseData, langUi);
        bufferMap.put(codebase, baseCheck);
        return baseCheck;
    }

}
