/* SctEngine - Copyright (c) 2016-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools;

import fr.exemole.sctserver.api.EngineContext;
import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctEngine;
import java.util.AbstractList;
import java.util.List;
import java.util.RandomAccess;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangPreferenceBuilder;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelHolder;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.db.api.ScrutariDBName;


/**
 *
 * @author Vincent Calame
 */
public final class EngineUtils {

    private EngineUtils() {

    }

    public static Lang getDefaultLangUi(ScrutariSession scrutariSession) {
        return getDefaultLangUi(scrutariSession.getEngine());
    }

    public static Lang getDefaultLangUi(SctEngine engine) {
        Lang langUi = engine.getEngineMetadata().getDefaultLangUi();
        if (langUi == null) {
            langUi = engine.getEngineContext().getWebappDefaultLangUi();
        }
        return langUi;
    }

    public static MessageLocalisation getMessageLocalisation(SctEngine engine, Lang langUi) {
        LangPreferenceBuilder langPreferenceBuilder = new LangPreferenceBuilder();
        if (langUi != null) {
            langPreferenceBuilder.addLang(langUi);
        }
        Lang defaultLangUi = engine.getEngineMetadata().getDefaultLangUi();
        if (defaultLangUi != null) {
            langPreferenceBuilder.addLang(defaultLangUi);
        }
        langPreferenceBuilder.addLang(engine.getEngineContext().getWebappDefaultLangUi());
        return engine.getEngineContext().getMessageLocalisation(langPreferenceBuilder.toLangPreference());
    }

    public static MessageLocalisation getMessageLocalisation(EngineContext engineContext, Lang langUi) {
        LangPreferenceBuilder langPreferenceBuilder = new LangPreferenceBuilder();
        if (langUi != null) {
            langPreferenceBuilder.addLang(langUi);
        }
        langPreferenceBuilder.addLang(engineContext.getWebappDefaultLangUi());
        return engineContext.getMessageLocalisation(langPreferenceBuilder.toLangPreference());
    }

    public static List<ScrutariDBName> wrap(ScrutariDBName[] array) {
        return new ScrutariDBNameList(array);
    }

    public static Label getTitleLabel(SctEngine engine, Lang lang) {
        LabelHolder labelHoder = engine.getEngineMetadata().getTitle();
        Label titleLabel = LabelUtils.getLangPartCheckedLabel(labelHoder, lang);
        if (titleLabel == null) {
            titleLabel = LabelUtils.getFirstLabel(labelHoder);
        }
        return titleLabel;
    }


    private static class ScrutariDBNameList extends AbstractList<ScrutariDBName> implements RandomAccess {

        private final ScrutariDBName[] array;

        private ScrutariDBNameList(ScrutariDBName[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public ScrutariDBName get(int index) {
            return array[index];
        }

    }

}
