/* UtilLib - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.attr;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
 *
 * @author Vincent Calame
 */
public class MutableAttributeHolder implements AttributeHolder {

    private final Map<AttributeKey, Attribute> attributeMap = new HashMap<AttributeKey, Attribute>();
    private final List<Attribute> attributeList = new ArrayList<Attribute>();

    @Override
    public Attribute getAttributeByKey(AttributeKey key) {
        return attributeMap.get(key);
    }

    @Override
    public int getAttributeCount() {
        return attributeList.size();
    }

    @Override
    public Attribute getAttributeByIndex(int i) {
        return attributeList.get(i);
    }

    public void initAttributes(AttributeHolder otherAttributeHolder) {
        attributeMap.clear();
        attributeList.clear();
        int count = otherAttributeHolder.getAttributeCount();
        for (int i = 0; i < count; i++) {
            Attribute attribute = otherAttributeHolder.getAttributeByIndex(i);
            attributeList.add(attribute);
            attributeMap.put(attribute.getAttributeKey(), attribute);
        }
    }

    public boolean removeAttribute(AttributeKey attributeKey) {
        Attribute attribute = attributeMap.remove(attributeKey);
        if (attribute != null) {
            attributeList.remove(attribute);
            return true;
        } else {
            return false;
        }
    }

    public boolean putAttribute(Attribute attribute) {
        AttributeKey attributeKey = attribute.getAttributeKey();
        if (attributeMap.containsKey(attributeKey)) {
            int size = attributeList.size();
            for (int i = 0; i < size; i++) {
                Attribute other = attributeList.get(i);
                if (other.getAttributeKey().equals(attributeKey)) {
                    if (AttributeUtils.isEqual(attribute, other)) {
                        return false;
                    }
                    attributeList.set(i, attribute);
                }
            }
        } else {
            attributeList.add(attribute);
        }
        attributeMap.put(attributeKey, attribute);
        return true;
    }

}
