/* UtilLib - Copyright (c) 2011-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.conditions;

import java.text.RuleBasedCollator;
import net.mapeadores.util.text.collation.CollationUnit;


/**
 *
 * @author Vincent Calame
 */
public class CollatedStringTestUnit {

    private final TextTest textTest;
    private final boolean isExcludingPartTestType;
    private final String searchCollatedString;

    private CollatedStringTestUnit(TextTest textTest, String searchCollatedString) {
        this.textTest = textTest;
        this.isExcludingPartTestType = ConditionsUtils.isExcludingPartTestType(textTest.getTestType());
        this.searchCollatedString = searchCollatedString;
    }

    public TextTest getTextTest() {
        return textTest;
    }

    public boolean accept(String collatedString) {
        short testType = textTest.getTestType();
        switch (testType) {
            case ConditionsConstants.EMPTY_TEST:
                return (collatedString.length() == 0);
            case ConditionsConstants.NOT_EMPTY_TEST:
                return (collatedString.length() > 0);
        }
        if (isExcludingPartTestType) {
            switch (testType) {
                case ConditionsConstants.NOT_MATCHES_TEST:
                    return !collatedString.equals(searchCollatedString);
                case ConditionsConstants.NOT_STARTSWITH_TEST:
                    return !collatedString.startsWith(searchCollatedString);
                case ConditionsConstants.NOT_ENDSWITH_TEST:
                    return !collatedString.endsWith(searchCollatedString);
                default:
                    return !collatedString.contains(searchCollatedString);
            }
        } else {
            switch (testType) {
                case ConditionsConstants.MATCHES_TEST:
                    return collatedString.equals(searchCollatedString);
                case ConditionsConstants.STARTSWITH_TEST:
                    return collatedString.startsWith(searchCollatedString);
                case ConditionsConstants.ENDSWITH_TEST:
                    return collatedString.endsWith(searchCollatedString);
                default:
                    return collatedString.contains(searchCollatedString);
            }
        }
    }

    public static CollatedStringTestUnit newInstance(TextTest textTest, RuleBasedCollator collator) {
        String searchCollatedString = CollationUnit.collate(textTest.getText(), collator);
        return new CollatedStringTestUnit(textTest, searchCollatedString);
    }

}
